<?php
// This file is part of the Studyplan plugin for Moodle
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.
/**
 * Edit/manage study plans
 * @package    local_treestudyplan
 * @copyright  2023 P.M. Kuipers
 * @license    https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once("../../config.php");

require_once($CFG->libdir.'/weblib.php');

use local_treestudyplan\contextinfo;
use \local_treestudyplan\courseservice;

$systemcontext = context_system::instance();

$PAGE->set_url("/local/treestudyplan/edit-plan.php", array());
require_login();

// Figure out the context (category or system, based on either category or context parameter).
$categoryid = optional_param('categoryid', 0, PARAM_INT); // Category id.
$contextid = optional_param('contextid', 0, PARAM_INT); // Context id.
if ($categoryid > 0) {
    $studyplancontext = context_coursecat::instance($categoryid);

} else if ($contextid > 0) {
    $studyplancontext = context::instance_by_id($contextid);
    if (in_array($studyplancontext->contextlevel, [CONTEXT_SYSTEM, CONTEXT_COURSECAT])) {
        $categoryid = $studyplancontext->instanceid;
    } else {
        $studyplancontext = $systemcontext;
    }
} else {
    // If no context is selected, find the first available context for this user.
    $availablecontexts = courseservice::list_accessible_categories_with_usage("edit");
    $contextid = 1; // Fallback to system context.
    if (count($availablecontexts) > 0) {
        $contextid = $availablecontexts[0]->ctxid;
    }
    // Reload page with selected category.
    $url = new \moodle_url('/local/treestudyplan/edit-plan.php', ["contextid" => $contextid]);
    header('Location: '.$url->out(false), true, 302);
    exit;
}

require_capability('local/treestudyplan:editstudyplan', $studyplancontext);
$ci = new contextinfo($studyplancontext);
$contextname = $ci->pathstr();

$PAGE->set_pagelayout('base');
//$PAGE->set_context($studyplancontext);
$PAGE->set_title(get_string('cfg_plans', 'local_treestudyplan')." - ".$contextname);
$PAGE->set_heading($contextname);

if ($studyplancontext->id > 1) {
    navigation_node::override_active_url(new moodle_url('/course/index.php', ['categoryid' => $categoryid ]));
    $PAGE->navbar->add(get_string('cfg_plans', 'local_treestudyplan'));
}

// Load javascripts and specific css.
$PAGE->requires->css(new moodle_url($CFG->wwwroot.'/local/treestudyplan/css/bootstrap-vue/bootstrap-vue.css'));
if ($CFG->debugdeveloper) {
    $PAGE->requires->css(new moodle_url($CFG->wwwroot.'/local/treestudyplan/css/devstyles.css'));
}
$PAGE->requires->js_call_amd('local_treestudyplan/page-edit-plan', 'init', [$studyplancontext->id, $categoryid, [
    "defaultAggregation" => get_config("local_treestudyplan","aggregation_mode"),
    "editMode" => $PAGE->user_is_editing()
]]);

$catlist = courseservice::list_accessible_categories_with_usage("edit");

/**
 * Shortcut function to provide translations
 *
 * @param mixed $str Translation key
 * @param null|string[] $param Parameters to pass to translation
 * @param string $plugin Location to search for translation strings
 * @return string Translation of key
 */
function t($str, $param = null, $plugin = 'local_treestudyplan') {
    print get_string($str, $plugin, $param);
}

print $OUTPUT->header();

?>
<div id='root'>
    <div class='vue-loader' v-show='false'>
        <div class="spinner-border text-primary" role="status">
            <span class="sr-only">Loading...</span>
        </div>
    </div>
    <div v-cloak>
        <div v-if='!activestudyplan && usedcontexts' class='ml-3 mb-3'>
            <b-form-select text='<?php print($contextname);?>' :value="contextid">
                <b-form-select-option v-for='ctx in usedcontexts' :key='ctx.id' :value="ctx.context_id" @click='switchContext(ctx)'
                    :active="ctx.context_id == contextid" :class="(ctx.studyplancount > 0) ? 'font-weight-bold' : ''"
                ><span v-for="(p, i) in ctx.category.path"
                    ><span v-if="i>0"> / </span>{{ p }}</span> <span>({{ ctx.studyplancount }})</span>
                </b-form-select-option>
            </b-form-select>
        </div>
        <h3 v-else><?php print $contextname; ?></h3>
        <div class="m-buttonbar" style="margin-bottom: 1em;">
            <a href='#' v-if='activestudyplan' @click.prevent='closeStudyplan'
                ><i style='font-size: 150%;' class='fa fa-chevron-left'></i> <?php t('back');?></a>
            <span v-if='activestudyplan'><?php t("studyplan_select"); ?></span>&nbsp;
            <b-form-select v-if='activestudyplan' lazy :text='dropdown_title' v-model='activestudyplan.id'>
                <b-form-select-option
                    v-for='(studyplan, planindex) in studyplans'
                    :value="studyplan.id"
                    :key='studyplan.id'
                    @click='selectStudyplan(studyplan)'>{{ studyplan.name }}</b-form-select-option>
            </b-form-select>&nbsp;
            <t-studyplan-edit
                @created="onStudyPlanCreated"
                v-if='!activestudyplan && !loadingstudyplan'
                mode="create"
                v-model="create.studyplan"
                type="button"
                variant="primary"
                default-aggregation="<?php print(get_config("local_treestudyplan","aggregation_mode")); ?>"
                :contextid='contextid'
            ><i class='fa fa-plus'></i> <?php t("studyplan_add");?></t-studyplan-edit>
            <b-button v-if='!activestudyplan && !loadingstudyplan'
                      variant='danger' href='#' role='presentation' @click="import_studyplan "
                ><i class='fa fa-upload'></i> <?php t("advanced_import_from_file");?></b-button>
        </div>

        <div class='t-studyplan-container'>
            <t-studyplan
                v-if='activestudyplan'
                v-model='activestudyplan'
                @moved="movedStudyplan"
                @toggletoolbox="toggletoolbox"
                :initeditmode="initialEditMode"
                @pagechanged="onPageChange"
            ></t-studyplan>
            <div v-else-if='loadingstudyplan' class="spinner-border text-primary" role="status">
                <span class="sr-only">Loading...</span>
            </div>
            <div v-else class='t-studyplan-notselected'>
                <p><?php t("studyplan_noneselected"); ?></p>
                <b-card-group deck>
                    <s-studyplan-card
                        v-for='(studyplan, planindex) in studyplans'
                        :key='studyplan.id'
                        v-model='studyplans[planindex]'
                        open
                        @open='selectStudyplan(studyplan)'
                        >
                        <template #title>
                            <span class='s-studyplan-card-title-buttons'>
                                <t-studyplan-edit v-model="studyplans[planindex]" @moved="movedStudyplan"></t-studyplan-edit>
                                <t-studyplan-associate v-model="studyplans[planindex]"></t-studyplan-associate>
                            </span>
                        </template>
                    </s-studyplan-card>
                </b-card-group>
            </div>
        </div>

        <b-sidebar
            id="toolbox-sidebar"
            :right='toolbox.right'
            shadow
            v-model="toolbox.shown"
            no-header
        >
            <div class="m-3 border-bottom-1 border-primary"><h3><?php t("toolbox")?></h3></div>
            <div class='t-toolbox-preface'>
                <b-form-checkbox v-model="toolbox.right" switch><?php t("toolbar-right");?></b-form-checkbox>
            </div>
            <b-tabs content-class='mt-3'>
                <b-tab title="<?php t('courses')?>">
                    <t-coursecat-list v-model="courses"></t-coursecat-list>
                </b-tab>
                <b-tab title="<?php t('flow')?>">
                    <ul class="t-toolbox">
                        <li><drag
                            :type="filterComponentType"
                            :data="{type: 'junction'}"
                            @cut=""
                            ><t-item-junction></t-item-junction><?php t("tool-junction") ?>
                            <template v-slot:drag-image="{data}"><t-item-junction></t-item-junction></template>
                        </drag></li>
                        <li><drag
                            :type="filterComponentType"
                            :data="{type: 'finish'}"
                            @cut=""
                            ><t-item-finish></t-item-finish><?php t("tool-finish") ?>
                            <template v-slot:drag-image="{data}"><t-item-finish></t-item-finish></template>
                        </drag></li>
                        <li><drag
                            :type="filterComponentType"
                            :data="{type: 'start'}"
                            @cut=""
                            ><t-item-start></t-item-start><?php t("tool-start") ?>
                            <template v-slot:drag-image="{data}"><t-item-start></t-item-start></template>
                        </drag></li>
                    </ul>
                </b-tab>
                <b-tab title="<?php t('badges')?>">
                    <b-card no-body class="mb-1">
                    <b-card-header header-tag="header" class="p-1" role="tab">
                        <b-button block v-b-toggle.relatedbadges variant="light">
                            <span class='float-left'><?php t("relatedbages",[],"badges") ?></span>
                            <span class="when-open float-right"><i class='fa fa-caret-down'></i></span>
                            <span class="when-closed float-right"><i class='fa fa-caret-right'></i></span>
                        </b-button>
                    </b-card-header>
                    <b-collapse id="relatedbadges" visible accordion="my-accordion" role="tabpanel">
                        <b-card-body>
                        <input v-model="filters.relatedbadges" @input="filter_relatedbadges" placeholder="<?php t("filter",[],"core"); ?>"></input>
                        &nbsp;<a @click="reset_relatedbadges" v-if="filters.relatedbadges" href='#'
                            ><i class='fa fa-times'></i></a>
                        <ul class="t-badges">
                            <li v-for="b in relatedbadges"><drag
                            class="t-badge-drag"
                            :type="filterComponentType"
                            :data="{type: 'badge', badge: b}"
                            @cut=""
                            ><img :class="(!b.active)?'disabled':''" :src="b.imageurl" :alt="b.name"> 
                            <span :class="(!b.active)?'disabled':''">{{b.name}}</span>
                            <template v-slot:drag-image="{data}"
                                    ><img :class="(!b.active)?'disabled':''" :src="b.imageurl" :alt="b.name"
                                ></template>
                            </drag></li>
                        </ul>
                        </b-card-body>
                    </b-collapse>
                    </b-card>
                    <b-card no-body class="mb-1">
                    <b-card-header header-tag="header" class="p-1" role="tab">
                        <b-button block v-b-toggle.systembadges variant="light">
                            <span class='float-left'><?php t("sitebadges",[],"badges") ?></span>
                            <span class="when-open float-right"><i class='fa fa-caret-down'></i></span>
                            <span class="when-closed float-right"><i class='fa fa-caret-right'></i></span>
                        </b-button>
                    </b-card-header>
                    <b-collapse id="systembadges" accordion="my-accordion" role="tabpanel">
                        <b-card-body>
                        <input v-model="filters.systembadges" @input="filter_systembadges" placeholder="<?php t("filter",[],"core"); ?>"></input>
                        &nbsp; <a @click="reset_systembadges" v-if="filters.systembadges" href='#'
                            ><i class='fa fa-times'></i></a>
                        <ul class="t-badges">
                            <li v-for="b in systembadges"><drag
                            class="t-badge-drag"
                            :type="filterComponentType"
                            :data="{type: 'badge', badge: b}"
                            @cut=""
                            ><img :class="(!b.active)?'disabled':''" :src="b.imageurl" :alt="b.name"> 
                                <span :class="(!b.active)?'disabled':''">{{b.name}}</span>
                                <template v-slot:drag-image="{data}"
                                    ><img :class="(!b.active)?'disabled':''" :src="b.imageurl" :alt="b.name"
                                ></template>
                            </drag></li>
                        </ul>
                        </b-card-body>
                    </b-collapse>
                    </b-card>
                    
                </b-tab>
            </b-tabs>
        </b-sidebar>
    </div>
</div>
<?php

print $OUTPUT->footer();
